
Servo penServo;

// setup servo
int servoPin  = 12;
int PEN_DOWN  = 10;    // angle of servo when pen is down
int PEN_UP    = 60;    // angle of servo when pen is up

float wheel_dia  =62;  //    # mm (increase = spiral out)
float wheel_dia_correction = 0.0; // positive spiral out; negative spiral in
float wheel_base =107; //    # mm (width wheel to wheel in mm) 
int steps_rev    =512; //    # 512 for 64x gearbox, 128 for 16x gearbox
int delay_time   =2;   //    # time between steps in ms

// Stepper sequence org->pink->blue->yel / IN4,IN2,IN1,IN3
int L_stepper_pins[] = {A3, A1, A0, A2};
int R_stepper_pins[] = {11, 8, 7, 10};

int fwd_mask[][4] =  {{1, 0, 1, 0},
                      {0, 1, 1, 0},
                      {0, 1, 0, 1},
                      {1, 0, 0, 1}};

int rev_mask[][4] =  {{1, 0, 0, 1},
                      {0, 1, 0, 1},
                      {0, 1, 1, 0},
                      {1, 0, 1, 0}};


bool isGO = false;




// ----- HELPER FUNCTIONS -----------
void initMotor()
{
  for(int pin=0; pin<4; pin++){
    pinMode(L_stepper_pins[pin], OUTPUT);
    digitalWrite(L_stepper_pins[pin], LOW);
    pinMode(R_stepper_pins[pin], OUTPUT);
    digitalWrite(R_stepper_pins[pin], LOW);
  }
}

void initMotorConfig(float _wheel_dia, float _wheel_base)
{
  wheel_dia = _wheel_dia;
  wheel_base = _wheel_base;
  
  initMotor();
}

void initServo(int _downAngle, int _upAngle)
{
    penServo.attach(servoPin);

    PEN_DOWN = _downAngle;
    PEN_UP = _upAngle;
}

int step(float distance){
  int steps = distance * steps_rev / ((wheel_dia + wheel_dia_correction) * 3.14159);
  /*
  Serial.print(distance);
  Serial.print(" ");
  Serial.print(steps_rev);
  Serial.print(" ");  
  Serial.print(wheel_dia);
  Serial.print(" ");  
  Serial.println(steps);
  delay(1000);*/
  return steps;  
}

void forward(float distance){
  int steps = step(distance);
  
  Serial.println(steps);
  
  for(int step=0; step<steps; step++){
    for(int mask=0; mask<4; mask++){
      for(int pin=0; pin<4; pin++){
        digitalWrite(L_stepper_pins[pin], rev_mask[mask][pin]);
        digitalWrite(R_stepper_pins[pin], fwd_mask[mask][pin]);
      }
      delay(delay_time);
    } 
  }
}


void backward(float distance){
  int steps = step(distance);
  for(int step=0; step<steps; step++){
    for(int mask=0; mask<4; mask++){
      for(int pin=0; pin<4; pin++){
        digitalWrite(L_stepper_pins[pin], fwd_mask[mask][pin]);
        digitalWrite(R_stepper_pins[pin], rev_mask[mask][pin]);
      }
      delay(delay_time);
    } 
  }
}


void right(float degrees){
  float rotation = degrees / 360.0;
  float distance = wheel_base * 3.14159 * rotation;
  int steps = step(distance);
  for(int step=0; step<steps; step++){
    for(int mask=0; mask<4; mask++){
      for(int pin=0; pin<4; pin++){
        digitalWrite(R_stepper_pins[pin], rev_mask[mask][pin]);
        digitalWrite(L_stepper_pins[pin], rev_mask[mask][pin]);
      }
      delay(delay_time);
    } 
  }   
}


void left(float degrees){
  float rotation = degrees / 360.0;
  float distance = wheel_base * 3.14159 * rotation;
  int steps = step(distance);
  for(int step=0; step<steps; step++){
    for(int mask=0; mask<4; mask++){
      for(int pin=0; pin<4; pin++){
        digitalWrite(R_stepper_pins[pin], fwd_mask[mask][pin]);
        digitalWrite(L_stepper_pins[pin], fwd_mask[mask][pin]);
      }
      delay(delay_time);
    } 
  }   
}


void done(){ // unlock stepper to save battery
  isGO = false;
  
  for(int mask=0; mask<4; mask++){
    for(int pin=0; pin<4; pin++){
      digitalWrite(R_stepper_pins[pin], LOW);
      digitalWrite(L_stepper_pins[pin], LOW);
    }
    delay(delay_time);
  }
}


void penup(){
  delay(250);
  Serial.println("PEN_UP()");
  penServo.write(PEN_UP);
  delay(250);
}


void pendown(){
  delay(250);  
  Serial.println("PEN_DOWN()");
  penServo.write(PEN_DOWN);
  delay(250);
}

void go(){
  int steps = step(5); // 10mm
  //isGO = true;
  
  Serial.println(steps);

  //if(isGO == true)
  {
  
    for(int step=0; step<steps; step++){
      for(int mask=0; mask<4; mask++){
        for(int pin=0; pin<4; pin++){
          digitalWrite(L_stepper_pins[pin], rev_mask[mask][pin]);
          digitalWrite(R_stepper_pins[pin], fwd_mask[mask][pin]);
        }
        delay(delay_time);
      } 
    }
    
  }
  //else
  {
    //done();
  }
}
